import { StructureElement, Unit } from '../../mol-model/structure';
import { ColorTheme } from '../../mol-theme/color';
import { ThemeDataContext } from '../../mol-theme/theme';
import { Color } from '../../mol-util/color';
import { ColorNames } from '../../mol-util/color/names';
import { ParamDefinition as PD } from '../../mol-util/param-definition';
import { Location } from '../../mol-model/location';
import { PDBTMDescriptor } from './types';

export type ResidueNameColorThemeParams = {
    pdbtmDescriptor: any
};

export function CustomColorTheme(
    ctx: ThemeDataContext,
    props: PD.Values<ResidueNameColorThemeParams>
): ColorTheme<ResidueNameColorThemeParams> {
    console.log('ColorTheme CTX', ctx);
    console.log('ColorTheme PROPS', props);

    return {
        factory: CustomColorTheme,
        granularity: 'group', //'residue' as any,
        color: getColor,
        props: props,
        description: 'TMDet...',
    };
}

const DefaultResidueColor = ColorNames.black;
const ColorMap = [
    Color(0xff0000),
    Color(0x0000ff),
    Color(0xffff00)
];

function getColor(location: Location): Color {
    let color = DefaultResidueColor;

    if (StructureElement.Location.is(location) && Unit.isAtomic(location.unit)) {
        const compId = StructureElement.residueIndex(location) % 3;
        color = ColorMap[compId]
    }
    return color;
}

export const CustomColorThemeProvider: ColorTheme.Provider<ResidueNameColorThemeParams, 'tmdet-custom-color-theme'> = {
    name: 'tmdet-custom-color-theme',
    label: 'TMDet Topology Theme',
    category: 'TMDet',
    factory: CustomColorTheme,
    getParams: () => ({ pdbtmDescriptor: { isOptional: true } }),
    defaultValues: { pdbtmDescriptor: undefined },
    isApplicable: () => true,
};
